@@ -2,7 +2,7 @@ import requests
 import json
 import re
 from dataclasses import dataclass
-from typing import Iterator, Generator
+from typing import Iterator, Generator, Union
 from itertools import islice
 
 from .dbc import DBC, http_json_res_parse
@@ -75,6 +75,15 @@ class ElasticSearch(DBC):
     def delete(self, index_pattern):
         return requests.delete(f'{self.base_url}/{index_pattern}', verify=False)
 
+    @http_json_res_parse
+    def create_or_cover(self, index: str, document: Union[str, dict], doc_id: str = None):
+        if isinstance(document, dict):
+            document = json.dumps(document)
+        return requests.post(
+            f'{self.base_url}/{index}/_doc/{doc_id if doc_id else ""}',
+            headers=_HEADERS, data=document
+        )
+
     @http_json_res_parse
     def _write(self, index, ndjson_data: str, timeout):
         return requests.post(
@@ -91,10 +100,12 @@ class ElasticSearch(DBC):
                 break
             write_result = self._write(index=index, ndjson_data=''.join(items), timeout=timeout)
             if write_result.get('errors') is True:
-                Logger.error('\n'.join([
-                                           str(item.get('index', {}).get('error', ''))
-                                           for item in write_result.get('items', [{}])
-                                       ][:10]))
+                Logger.error('\n'.join(
+                    [
+                        str(item.get('index', {}).get('error', ''))
+                        for item in write_result.get('items', [{}])
+                    ][:10]
+                ))
 
     def _check_template(self, index_pattern):
         url = f'{self.base_url}/_template/{TEMPLATE_NAME}'
@@ -143,9 +154,10 @@ class ElasticSearch(DBC):
         import numpy as np
         from pandas.core.dtypes.dtypes import DatetimeTZDtype
 
-        def to_es(inner_self: pd.DataFrame, index=None, index_col=None, numeric_detection=False,
-                  batch_size=10000, timeout=180, copy=True):
+        def to_es(inner_self: pd.DataFrame, index=None, index_col=None, id_col=None,
+                  numeric_detection=False, batch_size=10000, timeout=180, copy=True):
             _self = inner_self.copy() if copy else inner_self
+            _self['__$@_id'] = _self[id_col] if id_col else None
             if _self.empty:
                 return
 
@@ -169,8 +181,8 @@ class ElasticSearch(DBC):
                         lambda: _self[col].astype('float')
                     )()
             ndjsons = (
-                f"{json.dumps({'index': {'_index': name_tuple[0]}})}\n"
-                f"{json.dumps({col: value for col, value in zip(_self.columns, name_tuple[1:]) if self.not_na(value)})}\n"
+                f"{json.dumps({'index': {'_index': name_tuple[0], '_id': name_tuple[-1]}})}\n"
+                f"{json.dumps({col: value for col, value in zip(_self.columns, name_tuple[1:-1]) if self.not_na(value)})}\n"
                 for name_tuple in _self.itertuples()
             )
             return self._batch_write(
@@ -180,8 +192,7 @@ class ElasticSearch(DBC):
         def read_es(index, query_body: dict, batch_size=1000, timeout=180, total_size=None, log=False):
             return pd.DataFrame([
                 hit['_source']
-                for hit in
-                self.scroll_query(index, query_body, batch_size, timeout, total_size, log)['hits']['hits']
+                for hit in self.scroll_query(index, query_body, batch_size, timeout, total_size, log)['hits']['hits']
             ])
 
         pd.DataFrame.to_es = to_es
